#!/usr/bin/env python
'''
  #############################################################################
  #
  #     This Cplant(TM) source code is the property of Sandia National
  #     Laboratories.
  #
  #     This Cplant(TM) source code is copyrighted by Sandia National
  #     Laboratories.
  #
  #     The redistribution of this Cplant(TM) source code is subject to the
  #     terms of the GNU Lesser General Public License
  #     (see cit/LGPL or http://www.gnu.org/licenses/lgpl.html)
  #
  #     Cplant(TM) Copyright 1998, 1999, 2000, 2001, 2002 Sandia Corporation.
  #     Under the terms of Contract DE-AC04-94AL85000, there is a non-exclusive
  #     license for use of this work by or on behalf of the US Government.
  #     Export of this program may require a license from the United States
  #     Government.
  #
  #############################################################################

 File    : testbase.py
 Revision: $Revision: 1.3 $
 Author  : William McLendon, Narayan Desai
 Description:
        implementation of test drivers.
        Contains the following classes:
		tcpipTest
		SSSTest	 
		shellTest
		failTest    - just a dummy test, will always fail.
'''

import popen2
import socket
from time import sleep

# APITEST Framework Related
from testbase import Test,TODependencies,tInput,tOutput
from errorhandlers import TestSetupError

# ---------------------------------------------------------------------------------
# tcpipTest 
# ---------------------------------------------------------------------------------
class tcpipTest(Test):
    ''' class: tcpipTest '''
    __attrfields__=['name', 'desturl','destport', 'clearRecvBuf']
    __textfields__=[]				# REQUIRED 
    typemap={'dependencies':TODependencies,'output':tOutput,'input':tInput}

    def __setdefaults__(self):
        self.comm = "CLOSE"

    def setup(self):
        '''tcpipTest.setup()'''
        if hasattr(self,'prepared'): return
        self.prepared=1

    def execute(self,scratch):
        # Open the socket and execute
        self.destport = int(self.destport)
        s=socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        try:
            s.connect((self.desturl,self.destport))
        except socket.error,msg:
            raise TestSetupError, msg

        buffers = self.GetInput('send')
        for i in range(len(buffers)):
          s.send( buffers[i] )
        
        if self.expect.has_key('recv') or hasattr(self,'clearRecvBuf'):
            recvbuf = ""
            s.setblocking(0)
            fail = 0
            while fail < 10:
                try:
                    data = s.recv(1024)
                    recvbuf += data
                    fail = 0
                except socket.error, msg:
                    fail += 1
                    sleep(0.001)
            s.setblocking(1)
            if self.comm == 'CLOSE':
                s.close()
            sleep(0.01)
            if self.expect.has_key('recv'): 
                self.response['recv'] = recvbuf

# ---------------------------------------------------------------------------------
# SSSlib 
# ---------------------------------------------------------------------------------
class SSSTest(Test):
    """ class: SSSTest """
    __attrfields__=['name','destination']
    typemap={'dependencies':TODependencies,'output':tOutput,'input':tInput}
    responseSourcelist=['recv']

    def __setdefaults__(self):
        pass

    def display(self):
        print "D:\tname=(%s)"%self.name 
        print "D:\tdest=(%s)"%self.destination

    def setup(self):
        try:    
            from sss.ssslib import comm_lib, LookupError
            import sss.ssslib
        except: 
            print '***** ERROR :: IMPORT FAILED *****'
            print '\tpython could not import comm_lib from the sss.ssslib package,'
            print '\twhich is required for tests using ssslib.'
            exit(100)

        if hasattr(self,'prepared'): return
        for child in self.children:
            if child.tagname == 'send':
                self.smsg=child.__value__
                try:
                    self.scomm=child.comm
                except:
                    self.scomm="CLOSE"
        if self.options.has_key('debug'): self.display()
        self.comm=comm_lib()
        self.prepared=1

    def fail(self):
        raise TestFailure,(self.name,self.onMismatch)

    def execute(self,scratch):
        from sss.ssslib import LookupError
        try:
            h=self.comm.ClientInit(self.destination)
        except LookupError:
            raise TestSetupError, "Lookup Failure"
        buffers = self.GetInput('send')
        if self.options.has_key('verbose'): print 'V:\t\tSending Buffer: %s'%(''.join(buffers))
        self.comm.SendMessage(h,''.join(buffers))
        self.response['recv']=self.comm.RecvMessage(h)
        self.comm.ClientClose(h)
       
# ---------------------------------------------------------------------------------
# Shell Test Classes
# ---------------------------------------------------------------------------------
class shellTest(Test):
    ''' class ShellTest: object tree root containing command shell tests '''
    __attrfields__=['name','minPctMatch','maxPctMatch']
    typemap={'dependencies':TODependencies,'output':tOutput,'input':tInput}

    def setup(self): pass

    def execute(self,scratch):
        '''ShellTest.execute()'''
        # Execute command
        stdin = None
        try:    stdin = self.GetInput('stdin')
        except: pass

        if stdin:
            self.command = self.command + " < \""+stdin+"\""

        if self.options.has_key('verbose'):
            print "V:\tExecuting command = '%s'"%(self.command)

        child = popen2.Popen3(self.command, 1)
        child.wait()

        # Extract results (status, stdout, stderr)
        if self.expect.has_key('status'): self.response['status']= child.poll()/256
        if self.expect.has_key('stdout'): self.response['stdout']= ''.join(child.fromchild.readlines())
        if self.expect.has_key('stderr'): self.response['stderr']= ''.join(child.childerr.readlines())


# ---------------------------------------------------------------------------------------
class failTest(Test):
    __attrfields__=['name']
    typemap = {'dependencies':TODependencies}
    def setup(self):
        pass
    def execute(self,scratch):
        self.expect['foo']=[('REGEXP','a')]
        self.response['foo']=None

# ---------------------------------------------------------------------------------------
class passTest(Test):
    __attrfields__=['name']
    typemap = {'dependencies':TODependencies}
    def setup(self):
        pass
    def execute(self,scratch):
        self.expect['foo']=[('REGEXP','a')]
        self.response['foo']='a'

# ---------------------------------------------------------------------------------------
class httpTest(Test):
    __attrfields__=['name']
    typemap = {'dependencies':TODependencies}
    def setup(self):
        pass
    def execute(self,scratch):
        self.expect['a'] = [('REGEXP','a')]
        self.response['a'] = 'a'

# EOF
