#ifndef _SSSLIB_H
#define _SSSLIB_H

#ifdef __cplusplus
extern "C" {
#endif

#include<sys/types.h>
#include<sys/stat.h>

#ifdef __APPLE__
#include "/sw/include/expat.h"
#else
#include <expat.h>
#endif

#define DEBUG_TRANSPORT 1
#define DEBUG_AUTH 2
#define DEBUG_FRAMING 4
#define DEBUG_MANGLING 8

struct transport {
  void *state;
  void *handle;
  int (*cli_connect)(void *,char *,char *);
  int (*serv_connect)(void *,int);
  int (*write)(void *,void *,int);
  int (*writen)(void *,void *,int);
  int (*read)(void *,void *,int);
  int (*readn)(void *,void *,int);
  int (*close)(void *);
};

struct auth {
  void *state;
  void *handle;
  int (*cli_auth)(void *);
  int (*serv_auth)(void *);
  int (*close)(void *);
};

struct framing {
  void *state;
  void *handle;
  int (*send_message)(void *,void *,int);
  char *(*recv_message)(void *,int *);
  int (*close)(void *);
};

struct mangling {
  void *state;
  void *handle;
  char *(*mangle_message)(void *,char *,int,int *);
  char *(*unmangle_message)(void *,char *,int, int *);
  int (*close)(void *);
};

struct modules {
  int sss_errno;
  struct transport *transport;
  struct framing *framing;
  struct auth *auth;
  struct mangling *mangling;
};

  /* generic maximum number of stuff, arbitrary */

#define SSS_MAXPLEX 1024

  typedef struct SSS_ConfigDict_t {
    char protocol[128];
    char **keywords;
    char **values;
  } *SSS_ConfigDict;

  struct SSS_Protocol {
    int debug;
    int prot_errno;
    SSS_ConfigDict config;
    struct modules *modules;
    char *component;
    void *extraState; /*  */
  };

  /* represents another component */

  typedef struct SSS_Connection_t {
    char *protocol;
    void *libhandle;
    char *handle;
    char *host;
    char *extra;
    struct SSS_Protocol *state;
    char *(*recv)(struct SSS_Protocol *);
    int (*send)(struct SSS_Protocol *,char *);
    int (*cli_connect)(struct SSS_Protocol *,char *,char *);
    int (*serv_connect)(struct SSS_Protocol *,int);
    struct SSS_Protocol * (*getstate)(SSS_ConfigDict,int,int *);
    int (*close)(struct SSS_Protocol *);
      
  } *SSS_Connection;

  /* service cache */

  struct SSS_Service {
    char *protocol;
    char *service;
    char *extra;
    char *host;
    char *schema_version;
    char *tier;
    int valid;
  };

  typedef struct SSS_ServiceCache_t {
    struct SSS_Service *services[SSS_MAXPLEX];
  } *SSS_ServiceList;

  /* main structure everything will use */

  typedef struct SSS_t {
    char *sd_host;
    char *sd_protocol;
    char *sd_extra;
    char *sd_schema_version;
    char *sd_tier;
    int sss_errno;
    int debug;
    SSS_Connection connections[SSS_MAXPLEX];
    char *servprotocol;
    char *cluster;
    int numconfigs;
    int connection_index;
    char **protoprefs;
    char *msglog;
    struct SSS_ConfigDict_t *configs;
    SSS_ServiceList service_cache;
#ifdef __APPLE__
    struct timespec sss_conf_mtime;
#else
    time_t sss_conf_mtime;
#endif
  } *SSS;

  struct XML_Userdata {
    char reason[1024];
    int messageType;
    int success;
    int host;
    int extra;
    int protocol;
    int overwrite;
    SSS_ServiceList service_cache;
  };

  /* error values for sss->errno */

  enum {SSS_XML_PARSE=1,SSS_SD_FAIL,SSS_PROT_NAME_LONG,SSS_NO_SUCH_CONN,
	SSS_PROTO_NO_SUPPORT,SSS_FAM_NO_SUPPORT,SSS_NOMEM,SSS_NOSOCKMEM,SSS_INVALPROT,
	SSS_HOST_NOT_FOUND,SSS_NO_ADDRESS,SSS_UNRECOVERABLE_DNS_ERROR,SSS_TRY_AGAIN,
	SSS_BAD_FD,SSS_FAULT,SSS_ALREADY_CONNECTED,SSS_CONN_REFUSED,SSS_TIMEOUT,
	SSS_NET_UNREACH,SSS_ADDR_IN_USE,SSS_NO_FREE_PORTS,SSS_NO_CONNECT_PERM,
	SSS_NOSOCKPERM,SSS_NOT_A_SOCKET,SSS_SOCK_NOT_CONNECTED,SSS_INVALID_SIZE,
	SSS_INTERRUPT,SSS_INVAL_ARG,SSS_BROKEN_PIPE,SSS_AUTH_FAILED,SSS_LIBRARY_ERROR,
	SSS_NEED_MORE_INFO,SSS_RECV_TIMEOUT,SSS_CONN_RESET,SSS_PROT_ERROR,
	SSS_ILLEGAL_SEEK,SSS_SD_LOOKUP_ERROR,SSS_UNKNOWN_ERROR,SSS_PERMISSION_DENIED,
	SSS_NAMETOOLONG,SSS_FILENOTFOUND,SSS_NO_ROUTE,SSS_BAD_EXTRA,
	SSS_EOF};			/* this last one is important */
  
  /* error categories for exceptions in higher-level bindings */

  enum {SSS_INIT_ERROR,SSS_PARSE_ERROR,SSS_PROTOCOL_ERROR,SSS_SOCKET_ERROR,
	SSS_CONNECT_ERROR,SSS_COMMUNICATION_ERROR,SSS_INTERNAL_ERROR,SSS_LOOKUP_ERROR};

  /* message types for parsing, internal use */

  enum {SSS_REG_RESP,SSS_LOOKUP_RESP};
  
  /* max length of an error message */

  #define MAXERRLENGTH 1024

  /* functions */

  void SSS_SetServerProtocol(SSS sss,char *protocol);

  SSS_ServiceList SSS_StaticLookup(SSS sss,char *service);

  /* utility function for error descriptions */

  void SSS_strerror(int errnum,char *buf,int *type);

  /* utility function for error printing */

  void SSS_perror(int sss_errno,char *prefix);

  /* SSS_Init - reads config file, etc, malloc-age */

  SSS SSS_Init(char *file,int debug);

  /* SSS_SDRegister - registers with service directory */

  int SSS_SDRegister(SSS sss,char *service, char *host, int port, char *protocol,char *schema_version,char *tier);

  /* SSS_SDLookup - looks up a service */

  SSS_ServiceList SSS_SDLookup(SSS sss,char *service);

  /* SSS_SDRemove - Removes a component from the service directory */

  int SSS_SDRemove(SSS sss,char *service, char *host, int port,char *protocol);

  /* SSS_ServerInit - starts a server side end of a wire protocol */

  char *SSS_ServerInit(SSS sss,int extra);

  /* SSS_ClientInit - starts a client end of a wire protocol */

  char *SSS_ClientInit(SSS sss,char *service);

  /* SSS_ClientClose - closes a wire protocol connection */

  int SSS_ClientClose(SSS sss,char *handle);

  /* SSS_ServerClose - closes a wire protocol connection on the server end */

  int SSS_ServerClose(SSS sss,char *handle);

  /* SSS_SendMessage - sends a message */

  int SSS_SendMessage(SSS sss,char *handle,char *message);

  /* SSS_RecvMessage - receives a message */

  char *SSS_RecvMessage(SSS sss,char *handle);

  /* SSS_Close - deconstructs an SSS object */

  int SSS_Close(SSS sss);

  /* xml parsing handlers */

  void xml_starthandler(void *userData,const XML_Char *element_name,const XML_Char **attrs);
  void xml_endhandler(void *userData,const XML_Char *element_name);
  void xml_cdatahandler(void *userData);

  char *SSS_GetFromConfigDict(SSS_ConfigDict dictionary,char *keyword);

  SSS_ConfigDict SSS_GetGlobalConfigDict(SSS);
  char **SSS_GetProtocolPreference(SSS);

  void SSS_FreeConfigDict(SSS_ConfigDict);

#ifdef __cplusplus
}
#endif

#endif /* _SSSLIB_H */
